package de.ozgcloud.nachrichten.email;

import static de.ozgcloud.nachrichten.email.MailSendRequestTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import jakarta.mail.util.ByteArrayDataSource;
import jakarta.validation.Validation;
import jakarta.validation.Validator;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;

import de.ozgcloud.nachrichten.email.MailSendRequest.MailAttachment;

class MailSendRequestValidationTest {

	private final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

	@Nested
	class TestToAddressesValidation {
		@Test
		void shouldBeValid() {
			var request = MailSendRequestTestFactory.create();

			var violations = validator.validate(request);

			assertThat(violations).isEmpty();
		}

		@Test
		void shouldBeInvalidOnEmpty() {
			var request = MailSendRequestTestFactory.createBuilder().clearToAddresses().build();

			var violations = validator.validate(request);

			assertThat(violations).isNotEmpty();
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldBeInvalidOnBlankRecipientEmail(String email) {
			var recipient = MailRecipientTestFactory.createBuilder()
					.email(email)
					.build();
			var request = MailSendRequestTestFactory.createBuilder().clearToAddresses().toAddress(recipient).build();

			var violations = validator.validate(request);

			assertThat(violations).isNotEmpty();
		}
	}
}
