/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach;

import static de.ozgcloud.nachrichten.postfach.PostfachNachricht.ReplyOption.*;
import static de.ozgcloud.nachrichten.postfach.osi.MessageTestFactory.*;

import java.time.ZonedDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht.Direction;

public class PostfachNachrichtTestFactory {

	public static final String ID = UUID.randomUUID().toString();
	public static final String CREATED_AT_STR = "2020-04-01T10:30:10Z";
	public static final ZonedDateTime CREATED_AT = ZonedDateTime.parse(CREATED_AT_STR);
	public static final String CREATED_BY = UUID.randomUUID().toString();
	public static final String MAIL_BODY = "Body\nString";

	public static final String SENT_AT_STR = "2020-04-01T11:30:10Z";
	public static final ZonedDateTime SENT_AT = ZonedDateTime.parse(SENT_AT_STR);
	public static final boolean SENT_SUCCESSFUL = true;
	public static final String MESSAGE_CODE = PostfachMessageCode.SEND_SUCCESSFUL_MESSAGE_CODE.getMessageCode();
	public static final PostfachNachricht.ReplyOption REPLY_OPTION = MANDATORY;

	public static final PostfachNachricht.Direction DIRECTION = Direction.IN;

	public static final String ATTACHMENT_FILE_ID = "21";
	public static final List<String> ATTACHMENTS = List.of(ATTACHMENT_FILE_ID);

	public static PostfachNachricht create() {
		return createBuilder().build();
	}

	public static PostfachNachricht.PostfachNachrichtBuilder createBuilder() {
		return PostfachNachricht.builder()
				.id(ID)
				.referencedNachricht(GrpcPostfachMailTestFactory.REFERENCED_NACHRICHT_ID)
				.postfachAddress(PostfachAddressTestFactory.create())
				.messageId(MESSAGE_ID)
				.vorgangId(VORGANG_ID)
				.direction(DIRECTION)
				.createdAt(CREATED_AT)
				.createdBy(CREATED_BY)
				.sentAt(SENT_AT)
				.sentSuccessful(SENT_SUCCESSFUL)
				.messageCode(MESSAGE_CODE)
				.replyOption(REPLY_OPTION)
				.subject(SUBJECT)
				.mailBody(MAIL_BODY)
				.attachments(ATTACHMENTS);
	}

	public static Map<String, Object> asMap() {
		var map = new HashMap<String, Object>();
		map.put(PostfachNachricht.FIELD_ID, ID);
		map.put(PostfachNachricht.REFERENCED_NACHRICHT_FIELD, GrpcPostfachMailTestFactory.REFERENCED_NACHRICHT_ID);
		map.put(PostfachNachricht.POSTFACH_ADDRESS_FIELD, getPostfachAddressAsMap());
		map.put(PostfachNachricht.FIELD_MESSAGE_ID, MESSAGE_ID);
		map.put(PostfachNachricht.FIELD_VORGANG_ID, VORGANG_ID);
		map.put(PostfachNachricht.FIELD_DIRECTION, DIRECTION.name());
		map.put(PostfachNachricht.FIELD_CREATED_AT, CREATED_AT_STR);
		map.put(PostfachNachricht.FIELD_CREATED_BY, CREATED_BY);
		map.put(PostfachNachricht.FIELD_SENT_AT, SENT_AT_STR);
		map.put(PostfachNachricht.FIELD_SENT_SUCCESSFUL, SENT_SUCCESSFUL);
		map.put(PostfachNachricht.FIELD_MESSAGE_CODE, MESSAGE_CODE);
		map.put(PostfachNachricht.FIELD_REPLY_OPTION, REPLY_OPTION.name());
		map.put(PostfachNachricht.FIELD_SUBJECT, SUBJECT);
		map.put(PostfachNachricht.FIELD_MAIL_BODY, MAIL_BODY);
		map.put(PostfachNachricht.FIELD_ATTACHMENTS, ATTACHMENTS);
		return map;
	}

	private static Map<String, Object> getPostfachAddressAsMap() {
		return Map.of(PostfachAddress.TYPE_FIELD, PostfachAddressTestFactory.TYPE,
				PostfachAddress.VERSION_FIELD, PostfachAddressTestFactory.VERSION,
				PostfachAddress.IDENTIFIER_FIELD, getPostfachAddressIdentifierAsMap(),
				PostfachAddress.SERVICEKONTO_TYPE_FIELD, PostfachTestFactory.POSTFACH_TYPE);
	}

	private static Map<String, Object> getPostfachAddressIdentifierAsMap() {
		return Map.of(PostfachAddress.FIELD_POSTFACH_ID, PostfachAddressTestFactory.STRING_BASED_IDENTIFIER_POSTFACH_ID_VALUE);
	}
}
